#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List

from polyaxon.polyflow import V1ArtifactsMount
from polyaxon.schemas.types import V1ConnectionType


def override_stores(
    stores: List[V1ConnectionType], store_refs_by_name: Dict[str, V1ArtifactsMount]
) -> List[V1ConnectionType]:
    results = []
    if not stores or not store_refs_by_name:
        return results
    for store in stores:
        if store.name in store_refs_by_name:
            results.append(store.override(store_refs_by_name.get(store.name)))
    return results
